@@ -132,5 +132,9 @@ def format_coursecode(course):
 
     :return: prettified course code, such as "CS 61A"
     """
+    if course == "eecs189":
+        return "CS 189/289A"
+    if course == "cs189":
+        return "CS 189 (CDSS)"
     m = re.match(r"([a-z]+)([0-9]+[a-z]?)", course)
     return m and (m.group(1) + " " + m.group(2)).upper()
