@@ -18,6 +18,40 @@ def sh(
     cwd=None,
     inherit_env=True,
 ):
+    """Run a command on the command-line and optionally return output.
+
+    :param args: a variable number of arguments representing the command to
+        pass into :class:`~subprocess.Popen` or :func:`~subprocess.run`
+    :type args: *str
+
+    :param env: environment variables to set up the command environment with
+    :type env: dict
+
+    :param capture_output: a flag to return output from the command; uses
+        :class:`~subprocess.Popen`
+    :type capture_output: bool
+
+    :param stream_output: a flag to stream output from the command; uses
+        :func:`~subprocess.run`
+    :type stream_output: bool
+
+    :param quiet: a flag to run the command quietly; suppressed printed output
+    :type quiet: bool
+
+    :param shell: a flag to run the command in a full shell environment
+    :type shell: bool
+
+    :param cwd: the working directory to run the command in; current directory
+        is used if omitted
+    :type cwd: str
+
+    :param inherit_env: a flag to include :obj:`os.environ` in the environment;
+        ``True`` by default
+    :type inherit_env: bool
+
+    .. warning::
+        Only one of ``capture_output`` and ``stream_output`` can be ``True``.
+    """
     assert not (
         capture_output and stream_output
     ), "Cannot both capture and stream output"
@@ -47,8 +81,8 @@ def sh(
                 returncode = out.poll()
                 if returncode is not None:
                     if returncode != 0:
-                        # This exception will not be passed to the RPC handler, so we need
-                        # to handle it ourselves
+                        # This exception will not be passed to the RPC handler,
+                        # so we need to handle it ourselves
                         raise subprocess.CalledProcessError(returncode, args, "", "")
                     else:
                         return ""
@@ -63,8 +97,8 @@ def sh(
             args, env=env, stdout=subprocess.PIPE, shell=shell, cwd=cwd
         )
     if capture_output and not quiet:
-        print(out.stdout.decode("utf-8"), file=sys.stdout)
-        print(out.stderr.decode("utf-8"), file=sys.stderr)
+        print(out.stdout.decode("utf-8", "replace"), file=sys.stdout)
+        print(out.stderr.decode("utf-8", "replace"), file=sys.stderr)
     out.check_returncode()
     return out.stdout
 
@@ -83,8 +117,9 @@ def clean_all_except(folders: List[str]):
 def tmp_directory(*, clean=False):
     main_dir = os.getcwd()
     if clean:
-        sh("rm", "-rf", "tmp")
-        os.mkdir("tmp")
+        print("CREATING TEMP DIR", file=sys.stderr)
+        rmtree("tmp", ignore_errors=True)
+        os.makedirs("tmp", exist_ok=True)
     try:
         os.chdir("tmp")
         yield None
@@ -97,11 +132,11 @@ def redirect_descriptor(
     src: Union[IO, TextIO, BytesIO], target: Union[IO, TextIO, BytesIO]
 ):
     src_fd: int = src.fileno()
-    with os.fdopen(os.dup(src_fd), "wb") as saved:
+    alt_src_fd = os.dup(src_fd)
+    src.flush()
+    os.dup2(target.fileno(), src_fd)
+    try:
+        yield src
+    finally:
         src.flush()
-        os.dup2(target.fileno(), src_fd)
-        try:
-            yield src
-        finally:
-            src.flush()
-            os.dup2(saved.fileno(), src_fd)
+        os.dup2(alt_src_fd, src_fd)
